-- This file contains the VHDL description of the top level of the FFE

-- A known training sequence is assumed in this case which enables us to tune
-- the initial coefficients properly to avoid having large errors. (errors 
-- larger than 8 bits in magnitude may cause non convergence in this case.)
 
--Library declaraions
library IEEE;
use IEEE.Std_logic_1164.ALL;

library COMPASS_LIB;
use COMPASS_LIB.COMPASS.ALL;
--compass compile_off
--use COMPASS_LIB.COMPASS_ETC.ALL;
--compass compile_on

entity FFE is 
 port ( Clk    : in std_logic;
        Reset  : in std_logic;
        Iin    : in std_logic_vector(7 downto 0);      -- incoming i sample
        Qin    : in std_logic_vector(7 downto  0);     -- incoming q sample
        Ri     : in std_logic_vector(15 downto 0);     -- Local i reference 
        Rq     : in std_logic_vector(15 downto 0);     -- Local j reference 
        YI     : out std_logic_vector(15 downto 0);    -- output Q signal
        YQ     : out std_logic_vector(15 downto 0));   -- output Q signal
end FFE;        

architecture ffe of FFE is

component LMS
 port ( Clk    : in std_logic;
        Reset  : in std_logic;
        sign_i : in std_logic;
        sign_q : in std_logic;
        Ei     : in std_logic_vector(15 downto 0);  
        Eq     : in std_logic_vector(15 downto 0); 
        Ci     : out std_logic_vector(7 downto 0);
        Cq     : out std_logic_vector(7 downto 0));
end component;        

component FIR 
port (
  xin    : in std_logic_vector(7 downto 0);
  c0     : in std_logic_vector(7 downto 0);
  c1     : in std_logic_vector(7 downto 0);
  reset  : in std_logic; 
  clk    : in std_logic;
  yout   : out std_logic_vector(15 downto 0));
end component;

component diff port (
   clk: in std_logic;
   reset: in std_logic; 
   Ri : in std_logic_vector(15 downto 0);
   Rq : in std_logic_vector(15 downto 0);
   Yi : in std_logic_vector(15 downto 0);
   Yq : in std_logic_vector(15 downto 0);
   Ei : out std_logic_vector(15 downto 0);
   Eq : out std_logic_vector(15 downto 0) );
end component;

signal Ei : std_logic_vector(15 downto 0);
signal Eq : std_logic_vector(15 downto 0);

signal Yi_i : std_logic_vector(15 downto 0);
signal Yq_i : std_logic_vector(15 downto 0);

signal Ci_0 : std_logic_vector(7 downto 0);
signal Cq_0 : std_logic_vector(7 downto 0);
signal Ci_1 : std_logic_vector(7 downto 0);
signal Cq_1 : std_logic_vector(7 downto 0);

signal yi_reg : std_logic;
signal yq_reg : std_logic;


begin

yi <= yi_i;
yq <= yq_i;
 
lms0: lms port map (clk => clk,
                    reset => reset,
                    sign_i => Yi_i(15),
                    sign_q => Yq_i(15),
                    Ei => Ei,
                    Eq => Eq,
                    Ci => Ci_0,
                    Cq => Cq_0); 
          
lms1: lms port map (clk => clk,
                    reset => reset,
                    sign_i => Yi_reg,
                    sign_q => Yq_reg,
                    Ei => Ei,
                    Eq => Eq,
                    Ci => Ci_1,
                    Cq => Cq_1);


diff0: diff port map (clk => clk,
                      reset => reset,
                      Ri => Ri,
                      Rq => Rq,
                      Yi => Yi_i,
                      Yq => Yq_i,
                      Ei => Ei,
                      Eq => Eq ); 

ifir: fir port map (clk => clk,
                   reset => reset,
                   xin => Iin,
                   c0  => ci_0, 
                   c1  => ci_1, 
                   yout => yi_i);

qfir: fir port map (clk => clk,
                   reset => reset,
                   xin => Qin,
                   c0  => cq_0,
                   c1  => cq_1, 
                   yout => yq_i);

process (clk,reset)
 begin
  if reset = '0' then 
      yi_reg <= '0';
      yq_reg <= '0';
  elsif clk'event and clk ='1' then 
      yi_reg <= yi_i(15);
      yq_reg <= yq_i(15);
  end if;
 end process;
     


end ffe;
